/// \file FApp.h
/// \date 2007/12/01
/// \author Peter Dodds
/// \brief top level application class FRM::FApp
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _FRM_App_h_
#define _FRM_App_h_

#define __FRM

#include "FRM_Time.h"
#include "FRM_Status.h"
#include "FRM_Sound.h"

#include <deque>

namespace FRM
{
	/// \brief The main application class.
  /// To start the client; create a new application,
  /// and call FApp::Run()
	class FApp
	{
	public:
		//---------------------------------------
		/// \brief Default Constructor
    FApp();
    //---------------------------------------
    /// \brief Destructor
		virtual ~FApp();

    //---------------------------------------
    /// \brief initialise and start application
    void Run ();
		
  protected:
    //---------------------------------------
    /// \brief initialise application
    virtual void Init () = 0;

    //---------------------------------------
    /// \brief update application
    virtual void Update () = 0;

    //---------------------------------------
    /// \brief close application
    virtual void Close () = 0;


  public:
    Time time;
    Status status;

    unsigned _true_dt; ///< actual application dt
    unsigned dt; ///< world dt (able to pause)

    unsigned _avrg_dt;
    unsigned frameLength;

  protected:
    unsigned turn_number;
    unsigned sample_dt_size;
    std::deque<unsigned> sample_dt;
  };
}

#endif
