/// \file FFont.h
/// \date 2007/12/03
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _FRM_Font_h_
#define _FRM_Font_h_

#include <map>
#include <string>

namespace FRM
{
  /// \brief An individual character from the font file
  /// \note used internally by FRM::FFont
  class Font_Char
  {
    friend class FFont;
  private:
    // -----------------------------------
    /// \brief Constructor to set up all character info
    Font_Char (const char _c, const float& _uv0, const float& _uv1, const float& _uv2, const float& _uv3,
      const float& _x, const float& _y, const float& _w, const float& _h);
    // -----------------------------------
    /// \brief Destructor
    ~Font_Char ();

    char c; ///< character id
    float uv[4]; ///< uv coordinates on font-table image
    float x, y, w, h; ///< line offset (x,y) and normalised letter size
  };

  /// \brief A font set, from font-table file (.ft)
	class FFont
	{
  public:
    // -----------------------------------
    /// \brief Default Constructor
    FFont();
    // -----------------------------------
    /// \brief Constructor with font-table (.ft) file to load
    /// \param "const char * filename" .ft file
    FFont(const char* filename);
    // -----------------------------------
    /// \brief Destructor
		~FFont();
		
    // -----------------------------------
    /// \brief load font-table (.ft)
    /// \param "const char * filename" .ft file (default reloads file)
    /// \returns "bool" Load success
    bool Load (const char* filename = 0);

    bool Get (const char c, float& u00, float& v00, float& u11, float& v11,
      float& x_offset, float& y_offset, float& w, float& h);

    std::string name;
    std::string imgfile;
    std::string sourcefile;

    std::map<const char,Font_Char*> characters;
	};
}

#endif
