/// \file Input.h
/// \date 2007/12/03
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _FRM_Input_h_
#define _FRM_Input_h_

#include <MCG/Vector2.h>

#define FRM_K_TAB 0
#define FRM_K_RETURN 1
#define FRM_K_SPACE 2
#define FRM_K_RSHIFT 3
#define FRM_K_LSHIFT 4
#define FRM_K_RCTRL 5
#define FRM_K_LCTRL 6
#define FRM_K_RALT 7
#define FRM_K_LALT 8
#define FRM_K_UP 9
#define FRM_K_DOWN 10
#define FRM_K_LEFT 11
#define FRM_K_RIGHT 12

#define FRM_KEYSTATE_TABLE_MAX 13

#define FRM_M_BUTTON_1 0
#define FRM_M_BUTTON_2 1
#define FRM_M_BUTTON_3 2

#define FRM_E_QUIT 0
#define FRM_E_VIDEORESIZE 1
#define FRM_E_MOUSEMOTION 2
#define FRM_E_MOUSEBUTTONDOWN 3
#define FRM_E_MOUSEBUTTONUP 4
#define FRM_E_KEYDOWN 5
#define FRM_E_KEYUP 6

#define FRM_EVENT_TABLE_MAX 7

union SDL_Event;

namespace FRM 
{
  class FApp;

  struct Mouse
  {
    Mouse ();
    MCG::Vector2 p; ///< current position
    MCG::Vector2 m; ///< current motion
    bool buttonstate [3]; ///< on/off state of buttons
  };

  class FRM_InputMngr
  {
  public:
    FRM_InputMngr ();
    virtual ~FRM_InputMngr ();

    void Update ();
    bool EnableEvent (const unsigned& id);
    bool DisableEvent (const unsigned& id);

    Mouse mouse;
    char keystate [FRM_KEYSTATE_TABLE_MAX];

  protected:
    // default event functions:
    virtual void Quit ();
    virtual void VideoResize ();
    virtual void MouseMotion ();
    virtual void MouseButtonDown ();
    virtual void MouseButtonUP ();
    virtual void KeyDown ();
    virtual void KeyUP ();

    virtual void Check_KeyState ();

    SDL_Event* E;

  private:
    bool eventTable [FRM_EVENT_TABLE_MAX];
  };
}

#endif
