#ifdef _MSC_VER
#pragma once
#endif

#ifndef _FRM_Text_h_
#define _FRM_Text_h_

#include <string>
#include <vector>
#include <assert.h>

#include "FRM_DataType.h"
#include "FRM_Font.h"
#include <MCG/Vector3.h>

namespace FRM 
{
  class FTextManager;

  class FText
  {
    friend class FTextManager;
  private: 
    FText ();
    ~FText ();

    void Update ();

  public:
    void* ptr;
    DataType dtype;
    std::string text;
    int x, y;
    int size;
    int layer; // 0 - 10
    int life; // 0 - infinite

    MCG::Colour3 color;
    FFont* font;
  };

  class FTextManager
  {
  public:
    FTextManager ();

    ~FTextManager ();

    // -----------------------------------
    /// \brief Update the text on screen
    /// \param unsigned dt - change in time since last redisplay
    void Update (unsigned dt);

    // -----------------------------------
    /// \brief adds text to the scene
    /// \param const int x - screen position in x
    /// \param const int y - screen position in y
    /// \param const unsigned life - time on screen (0 = forever)
    /// \param const int size - size on screen
    /// \param const char * s - string to be printed
    /// \param const int layer - drawing layer to 
    /// \returns FText* - pointer to the new text
    FText* print (const int x, const int y, const unsigned life, const int size, const char* fontFile, const char* s, const int layer = 10);
    
    // -----------------------------------
    /// \brief adds text to the scene
    /// \param const int x - screen position in x
    /// \param const int y - screen position in y
    /// \param const unsigned life - time on screen (0 = forever)
    /// \param const int size - size on screen
    /// \param const DataType dtype - data type
    /// \param const void * ptr - data to be printed
    /// \param const int layer - drawing layer to 
    /// \returns FText* - pointer to the new text
    FText* print (const int x, const int y, const unsigned life, const int size, const char* fontFile, const DataType dtype, void* ptr, const int layer = 10);

    // -----------------------------------
    /// \brief safely remove FText from the scene
    /// \param FText * T - FText to be deleted
    /// \returns bool - if the FText was deleted
    bool erase (const FText* T);

  private:
    FFont* LoadFont (const char* fontFile);

  public:
    std::vector<FText*> text;

  private:
    std::vector<FFont*> fonts;
  };
}

#endif
