/// \file FRM_Transform.h
/// \date 2008/01/07
/// \author Peter Dodds
/// \brief
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _FRM_FRM_Transform_h_
#define _FRM_FRM_Transform_h_

#include <MCG/Vector3.h>
#include <MCG/Quaternion.h>
#include <MCG/Matrix.h>

namespace FRM
{
	/// \brief A base class for all 3d space objects
	class FTransform
	{
	public:
		//---------------------------------------
		/// \brief Default Constructor
    FTransform();
		//---------------------------------------
		/// \brief Destructor
		virtual ~FTransform();

    virtual void Rotate (const MCG::Quaternion& rot, const bool relative = 0);
    void RotateManual (const MCG::Real& angle, const MCG::Real& _x, const MCG::Real& _y, const MCG::Real& _z, const bool relative = 0);
    virtual void Translate (const MCG::Vector3& tra, const bool relative = 0);
    void TranslateManual (const MCG::Real& _x, const MCG::Real& _y, const MCG::Real& _z, const bool relative = 0);

    void SetRotation (const MCG::Quaternion& rot);
    void SetTranslation (const MCG::Vector3& tra);

    MCG::Quaternion GetRotation ();
    MCG::Vector3 GetTranslation ();

    void EnablePivot ();
    void DisablePivot ();

    virtual void TranslatePivot (const MCG::Vector3& tra, const bool relative = 0);
    void TranslatePivotManual (const MCG::Real& _x, const MCG::Real& _y, const MCG::Real& _z, const bool relative = 0);

    void SetPivot (const MCG::Vector3& piv, const bool relative = 0);
    MCG::Vector3 GetPivot (const bool relative = 0);

    void SetMatrix (const MCG::Matrix& mat);
    MCG::Matrix GetMatrix ();

    virtual void CalculateMatrix ();

  protected:
    bool isCalculated;
    bool isPivot;

    // the pivot is in local space, so is relative to all rotations and translations
    MCG::Vector3 m_piv; ///< local space pivot position

    // transforms
    MCG::Quaternion m_rot; ///< world space rotation
    MCG::Vector3 m_tra; ///< world space translation

    // transform matrix
    MCG::Matrix m_mat;
	};
}

#endif
