#ifdef _MSC_VER
#pragma once
#endif

#ifndef _MCG_Def_h_
#define _MCG_Def_h_

#include <math.h>

namespace MCG
{
  /// A real number (eg. float)
#ifdef MCG_DOUBLE_ACCURACY 
  typedef double Real;
  #define MCG_SIN(X) sin(X)
  #define MCG_COS(X) cos(X)
  #define MCG_ACOS(X) acos(X)
  #define MCG_SQRT(X) sqrt(X)
#else
  typedef float Real;
  #define MCG_SIN(X) sinf(X)
  #define MCG_COS(X) cosf(X)
  #define MCG_ACOS(X) acosf(X)
  #define MCG_SQRT(X) (float)(sqrt(X))
#endif

  ///	This is the epsilon value used for floating point comparisons
  const Real g_vecEpsilon = 0.001f;

  ///	PI / 180 constant
  const Real PI_OVER_180 = 0.017453292519943295769236907684886f;

  ///	PI constant
  const Real PI          = 3.1415926535897932384626433832795f;
}
#ifdef FCOMPARE
  #undef FCOMPARE
#endif

#ifdef DEG2RAD
  #undef DEG2RAD
#endif

#define FCOMPARE(X_,Y_) (((X_)>((Y_) - MCG::g_vecEpsilon)) && ( (X_) < ((Y_)+ MCG::g_vecEpsilon)))
#define DEG2RAD(X) (MCG::PI_OVER_180 * X)


#endif
