/// \file Quaternion.h
/// \date 2007/12/07
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _MCG_Quaternion_h_
#define _MCG_Quaternion_h_

#include <iostream>
#include "Def.h"

namespace MCG
{
  struct Matrix;

#pragma pack(push,1)
  struct Quaternion
  {
  public:
    Quaternion(const Real& _x = 0, const Real& _y = 0, const Real& _z = 0, const Real& _w = 1);
    Quaternion (const Quaternion& q);
    Quaternion (Matrix& m);

    const Quaternion& Set (const Real& _x = 0, const Real& _y = 0, const Real& _z = 0, const Real& _w = 1);
    const Quaternion& SetAxisAngle (const Real& angle, const Real& _x, const Real& _y, const Real& _z );

    Matrix ToMatrix ();
    void ToAxisAngles (Real& _x, Real& _y, Real& _z);
    
    const Quaternion& Normalise ();
    Quaternion Normalised();

    MCG::Real MagSquared ();
    MCG::Real Mag ();


    Quaternion operator- ();

    Quaternion operator* (const Quaternion& q);

    const Quaternion& operator= (const Quaternion& q);
    const Quaternion& operator= (Matrix& m);

#if 0
    const Quaternion& operator *= (const Quaternion& q);
#endif // 0

    bool operator== (const Quaternion& q);
    bool operator!= (const Quaternion& q);

    friend std::ostream& operator << (std::ostream& os,const Quaternion &q)
    {
      return os << q.x << " " << q.y << " " << q.z << " " << q.w;
    }

    friend Quaternion SLERP (const Quaternion& q1, const Quaternion& q2, const Real& t);

    union 
    {
      Real m_data[4];
      struct 
      {
        Real x;
        Real y;
        Real z;
        Real w;
      };
    };
  };
#pragma pack(pop)

  Quaternion DIFF (const Quaternion& orig, const Quaternion& dest);
}

#endif
