/// \file Vector2.h
/// \date 2007/11/13
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _MCG_Vector2_h_
#define _MCG_Vector2_h_

#include "Def.h"
#include <ostream>

namespace MCG
{
  struct Vector3;
  struct Vector4;

	/// \brief
#pragma pack(push,1)
	struct Vector2
	{
    Vector2();
    Vector2(const Real _x, const Real _y);
    Vector2(const Vector2& v);
    Vector2(const Vector3& v);
    //Vector2(const Vector4& v);
    ~Vector2();

    const Vector2& Set (const Real _x = 0, const Real _y = 0);

    Real LengthSquared ();
    Real Length ();
    const Vector2& Normalise ();
    const Vector2& Scale( const Real s );

    Vector2 operator - () const;

    Vector2 operator + (const Vector2& v) const;
    Vector2 operator - (const Vector2& v) const;
    Real operator * (const Vector2& v) const;
    Vector2 operator ^ (const Vector2& v) const;
    Vector2 operator * (Real f) const;
    Vector2 operator / (Real f) const;

    const Vector2& operator = (const Vector2& v);
    const Vector2& operator = (const Vector3& v);
    //const Vector2& operator = (const Vector4& v);

    const Vector2& operator += (const Vector2& v);
    const Vector2& operator -= (const Vector2& v);
    const Vector2& operator *= (const Real f);
    const Vector2& operator /= (const Real f);

    bool operator == (const Vector2& v) const;
    bool operator != (const Vector2& v) const;

    friend std::ostream& operator << (std::ostream &s, const Vector2& v);

    union
    {
      struct
      {
        Real x, y;
      };
      Real m_data[2];
    };
	};
#pragma pack(pop)
}

#endif
