/// \file Vector3.h
/// \date 2007/11/13
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------
#ifdef _MSC_VER
#pragma once
#endif

#ifndef _MCG_Vector3_h_
#define _MCG_Vector3_h_

#include "Def.h"
#include <ostream>

namespace MCG
{
  struct Vector2;

#pragma pack(push,1)
  struct Vector3
  {
    Vector3();
    Vector3(const Real _x, const Real _y, const Real _z);
    Vector3(const Vector2& v);
    Vector3(const Vector3& v);
    //Vector3(const Vector4& v);
    ~Vector3();

    const Vector3& Set (const Real _x = 0, const Real _y = 0, const Real _z = 0);

    Real LengthSquared ();
    Real Length ();
    const Vector3& Normalise ();
    const Vector3& Scale (Real s);

    Vector3 operator - () const;

    Vector3 operator + (const Vector3& v) const;
    Vector3 operator - (const Vector3& v) const;
    Real operator * (const Vector3& v) const;
    Vector3 operator ^ (const Vector3& v) const;
    Vector3 operator * (Real f) const;
    Vector3 operator / (Real f) const;

    const Vector3& operator = (const Vector2& v);
    const Vector3& operator = (const Vector3& v);
    //const Vector3& operator = (const Vector4& v);

    const Vector3& operator += (const Vector3& v);
    const Vector3& operator -= (const Vector3& v);
    const Vector3& operator *= (const Real f);
    const Vector3& operator /= (const Real f);

    bool operator == (const Vector3& v) const;
    bool operator != (const Vector3& v) const;
    
    friend std::ostream& operator << (std::ostream &s, const Vector3& v);
    
    union
    {
      struct
      {
        Real x, y, z;
      };
      struct
      {
        Real r, g, b;
      };
      Real m_data[3];
    };

  };
#pragma pack(pop)

typedef Vector3 Colour3;
}

#endif
