/// \file Vector4.h
/// \date 2007/11/18
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2007 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _MCG_Vector4_h_
#define _MCG_Vector4_h_

#include "Def.h"
#include <ostream>

namespace MCG
{
  struct Vector2;
  struct Vector3;

  /// \brief
#pragma pack(push,1)
	struct Vector4
	{
    Vector4();
    Vector4(const Real _x, const Real _y, const Real _z, const Real _w);
    Vector4(const Vector2& v);
    Vector4(const Vector3& v);
    Vector4(const Vector4& v);
    ~Vector4();

    const Vector4& Set (const Real _x = 0, const Real _y = 0, const Real _z = 0, const Real _w = 0);

    Vector4 operator - () const;

    Vector4 operator + (const Vector4& v) const;
    Vector4 operator - (const Vector4& v) const;
    Real operator * (const Vector4& v) const;
    Vector4 operator * (Real f) const;
    Vector4 operator / (Real f) const;

    const Vector4& operator = (const Vector2& v);
    const Vector4& operator = (const Vector3& v);
    const Vector4& operator = (const Vector4& v);

    const Vector4& operator += (const Vector4& v);
    const Vector4& operator -= (const Vector4& v);
    const Vector4& operator *= (const Real f);
    const Vector4& operator /= (const Real f);

    bool operator == (const Vector4& v) const;
    bool operator != (const Vector4& v) const;

    friend std::ostream& operator << (std::ostream &s, const Vector4& v);

    union
    {
      struct
      {
        Real x, y, z, w;
      };
      struct
      {
        Real r, g, b, a;
      };
      Real m_data[4];
    };
  };
#pragma pack(pop)

  typedef Vector4 Colour4;
}

#endif
