/// \file AiAgent.cpp
/// \date 2008/01/24
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#include "AiAgent.h"
#include "AiMove.h"

#include "AiPath.h"
#include "AiPathNode.h"
#include "AiManager.h"
 
#include <assert.h>

namespace BUG
{
  unsigned AiAgent::_nextID = 1; // set _nextID to start condition
	

  //------------------------------------------------ AiAgent::AiAgent
  AiAgent::AiAgent()
    : m_id (GetUniqueID())
  {
    // check the AiManager has been created
    assert(AiManager::Instance());

    // register AiAgent with the AiManager
    AiManager::Instance()->m_agents[m_id] = this;
  }

  //------------------------------------------------ AiAgent::~AiAgent
  AiAgent::~AiAgent()
  {
    std::vector<AiMove*>::iterator it = m_move_q.begin();
    for (;it!=m_move_q.end();++it)
      delete (*it);

    AiManager::Instance()->m_agents.erase(m_id);
  }

  //------------------------------------------------ AiAgent::Ai_UsePathAsMoves
  void AiAgent::Ai_UsePathAsMoves( const AiPath& path, const BUGu8& op /*= k_Replace*/ )
  {
    if (op == k_Replace)
    {
      std::vector<AiMove*>::iterator it = m_move_q.begin();
      for (;it!=m_move_q.end();++it)
        delete (*it);
      m_move_q.clear();
      m_move_age = 0;
    }

    if (op == k_Replace || op == k_InsertEnd)
    {
      for (unsigned i=0; i!=path.size(); ++i)
      {
        BUG::AiMove_Goto* order = (BUG::AiMove_Goto*)Ai_NewMove(BUG_Move_Goto);
        order->goal.x = path[i]->m_pos.x; order->goal.y = path[i]->m_pos.y;
        m_move_q.push_back(order);
      }
    }
    else
    {
      m_move_age = 0;
      for (unsigned i=path.size(); i!=0; ++i)
      {
        unsigned id = i-1;
        BUG::AiMove_Goto* order = (BUG::AiMove_Goto*)Ai_NewMove(BUG_Move_Goto);
        order->goal.x = path[id]->m_pos.x; order->goal.y = path[id]->m_pos.y;

        m_move_q.insert( m_move_q.begin(), order);
      }
    }
  }

  //------------------------------------------------ AiAgent::Ai_Update
  void AiAgent::Ai_Update( const unsigned dt )
  {
    if (!m_move_q.empty())
    {
      bool delete_me = Ai_ProcessMove(m_move_q[0], dt);
      m_move_age += dt;

      if ( delete_me || (m_move_q[0]->m_life != 0 && m_move_q[0]->m_life < dt) )
      {
        delete m_move_q[0];
        m_move_q.erase(m_move_q.begin());
      }
    }
  }

  //------------------------------------------------ AiAgent::Ai_AddOrder
  AiMove* AiAgent::Ai_AddMove( const BUGu8& type, const BUGu8& op /*= k_InsertEnd*/ )
  {
    AiMove* move = Ai_NewMove(type);
    switch(op)
    {
    case k_InsertStart:
      m_move_q.insert( m_move_q.begin(), move);
      m_move_age = 0;
      break;
    case k_Replace:
      {
        std::vector<AiMove*>::iterator it = m_move_q.begin();
        for (;it!=m_move_q.end();++it)
          delete (*it);
        m_move_q.clear();

        m_move_q.push_back(move);
        m_move_age = 0;
      }
      break;

    case k_InsertEnd:
    default:
      m_move_q.push_back(move);
      break;
    }
    return move;
  }

  //------------------------------------------------ AiAgent::Ai_ProcessMove
  bool AiAgent::Ai_ProcessMove( AiMove* _o, const unsigned dt )
  {
    bool delete_me = false;

    switch(_o->GetType())
    {
    case BUG_Move_Goto:
      {
        AiMove_Goto* move = (AiMove_Goto*)_o;
        Real x, y;
        GetTranslation(x,y);
        
        x = move->goal.x - x;
        y = move->goal.y - y;

        double tmp = x*x + y*y;

        // at the destination
        if (tmp < 4)
        {
          delete_me = true;
          break;
        }

        Real speed = 0.01f;
        if(tmp>0.0) 
        {
          tmp = speed*dt / BUG_SQRT(tmp);
          x *= (Real)tmp;
          y *= (Real)tmp;
        }


        // move the agent
        Translate(x,y);
      }
      break;
    default:
      break;
    }

    return delete_me;
  }

  //------------------------------------------------ AiAgent::Ai_Order_New
  AiMove* AiAgent::Ai_NewMove( BUGu8 type )
  {
    AiMove* _r;
    switch(type)
    {
    case BUG_Move_Goto:
      _r = new AiMove_Goto;
      break;
    default:
      _r = 0;
      break;
    }

    return _r;
  }
}