/// \file AiDef.cpp
/// \date 2008/03/07
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#include "AiDef.h"

namespace BUG
{
	//------------------------------------------------ GetPointOfIntersection
  BUG::Vector GetPointOfIntersection( const Vector& a1, const Vector& a2, const Vector& b1, const Vector& b2 )
  {
    /**
    detecting-line-to-line-intersection taken from http://processinghacks.com/hacks:detecting-line-to-line-intersection
    @author Ryan Alexander
    */

    // Infinite Line Intersection

    /*
    float bx = x2 - x1;
    float by = y2 - y1;
    float dx = x4 - x3;
    float dy = y4 - y3;

    float b_dot_d_perp = bx*dy - by*dx;

    if(b_dot_d_perp == 0) return null;

    float cx = x3-x1;
    float cy = y3-y1;

    float t = (cx*dy - cy*dx) / b_dot_d_perp;

    return Vector2f(x1+t*bx, y1+t*by);
    */    

    Vector A (a2.x - a1.x, a2.y - a1.y);
    Vector B (b2.x - b1.x, b2.y - b1.y);

    Real perp_AdotB = A.x*B.y - A.y*B.x;

    if (perp_AdotB == 0)
      return 0;

    Vector C (b1.x - a1.x, b1.y - a1.y); 
    float t = (C.x*B.y - C.y*B.x) / perp_AdotB;

    C.x = a1.x + t*B.x; 
    C.y = a1.y + t*B.y;
    return C;
  }

  //------------------------------------------------ DistanceToLine
  BUG::Real DistanceToLine( const Vector& lineA, const Vector& lineB, const Vector& pnt )
  {
    Vector normal ( -(lineB.y - lineA.y), lineB.x - lineA.x );
    Real mag = BUG_SQRT(normal.x * normal.x + normal.y * normal.y);
    if (mag == 0)
      return 0;
    mag = 1/mag;

    normal.x = normal.x*mag; normal.y = normal.y*mag;

    return (pnt.x - lineA.x) * normal.x + (pnt.y - lineA.y) * normal.y;
  }
}