#ifdef _MSC_VER
#pragma once
#endif

#ifndef _BUG_AiDef_h_
#define _BUG_AiDef_h_

#include <math.h>


#ifdef BUG_64BIT
  //need to change these! not really 64bit compatible
  typedef unsigned char BUGu8;
  typedef unsigned int BUGu32;
  typedef unsigned long BUGu64;
#else
  typedef unsigned char BUGu8;
  typedef unsigned int BUGu32;
  typedef unsigned long BUGu64;
#endif

namespace BUG
{
  /// A real number (eg. float)
#ifdef BUG_DOUBLE_ACCURACY 
  typedef double Real;
#define BUG_SIN(X) sin(X)
#define BUG_COS(X) cos(X)
#define BUG_ACOS(X) acos(X)
#define BUG_SQRT(X) sqrt(X)
#else
  typedef float Real;
#define BUG_SIN(X) sinf(X)
#define BUG_COS(X) cosf(X)
#define BUG_ACOS(X) acosf(X)
#define BUG_SQRT(X) (float)(sqrt(X))
#endif

#pragma pack(push,1)
  /// \brief small class to hold a vector defined by x and y
  struct Vector
  {
    // -----------------------------------
    /// \brief Constructor that sets by default x and y to 0
    /// \param "const Real & _x" x value
    /// \param "const Real & _y" y value
    Vector (const Real& _x = 0, const Real& _y = 0)
      : x (_x), y (_y)
    {}

    // -----------------------------------
    /// \brief copy operator
    /// \param "const Vector & v" the data to be assigned
    /// \returns "const Vector&" the value of this class after the operation
    const Vector& operator = (const Vector& v)
    {
      x = v.x; y = v.y;
      return *this;
    }

    Real x, y; ///< the vector data
  };
#pragma pack(pop)  



  ///	This is the epsilon value used for floating point comparisons
  const Real g_vecEpsilon = 0.001f;

  ///	PI / 180 constant
  const Real PI_OVER_180 = 0.017453292519943295769236907684886f;

  ///	PI constant
  const Real PI          = 3.1415926535897932384626433832795f;

  Real DistanceToLine (const Vector& lineA, const Vector& lineB, const Vector& pnt);
  Vector GetPointOfIntersection (const Vector& a1, const Vector& a2, const Vector& b1, const Vector& b2);

}
#ifndef FCOMPARE
  #define FCOMPARE(X_,Y_) (((X_)>((Y_) - BUG::g_vecEpsilon)) && ( (X_) < ((Y_)+ BUG::g_vecEpsilon)))
#endif // FCOMPARE

#ifndef DEG2RAD
  #define DEG2RAD(X) (BUG::PI_OVER_180 * X)
#endif // DEG2RAD

#endif
