/// \file AiManager.h
/// \date 2008/02/17
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#ifdef _MSC_VER
#pragma once
#endif

#ifndef _BUG_AiManager_h_
#define _BUG_AiManager_h_

#include <map>
#include "AiPathfinder.h"

namespace BUG
{
  class AiAgent;
  class AiBlock;

  /// \brief Singleton Manager. This is the centre to the AI library, and is required to work
  class AiManager
  {
  friend class AiAgent;

  public:
    // -----------------------------------
    /// \brief Get an instance of the AiManager singleton
    /// \returns "AiManager*" pointer to this class
    static AiManager* Instance() {
      return m_singleton;
    }

    // -----------------------------------
    /// \brief Create the instance of the AiManager singleton
    /// \returns "AiManager*" pointer to this class
    static AiManager* Make()
    {
      if (m_singleton)
        return 0;
      m_singleton = new AiManager;
      return m_singleton;
    }

    // -----------------------------------
    /// \brief free the memory associated to the AiManager safely
    static void Delete ()
    {
      if (m_singleton){
        delete m_singleton;
        m_singleton = 0;
      }
    }


    // -----------------------------------
    /// \brief Update each of the agents
    /// \param "unsigned dt" change in time since last update.
    virtual void Update (unsigned dt);

  protected:
    //---------------------------------------
    /// \brief Default Constructor
    AiManager() {}
    //---------------------------------------
    /// \brief Destructor
    ~AiManager() {}


    std::map<unsigned,AiAgent*> m_agents; ///< tracked agents in game
    //std::map<unsigned,AiBlock*> m_blocks;

    static AiManager* m_singleton; ///< pointer to this variables

  public:
    AiPathfinder m_pathfinder; ///< pathfinding component
  };
}

#endif