/// \file Application.cpp
/// \date 2008/01/24
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#include "Application.h"

#include <stdlib.h>
#include <assert.h>
#include "Instance.h"
#include <BUG/AiPathfinder.h>
#include <BUG/AiPathNode.h>
#include <MCG/Def.h>

#include <DebugUtl.h>


#include <map>
#include <BUG/AiPath.h>
#include "Unit.h"

namespace Roach
{

  //------------------------------------------------ Application::Application
  Application::Application()
    : m_mode (App_Mode_Play)
  {

  }

  //------------------------------------------------ Application::Init
  void Application::Init()
  {
    m_render.fullscreen = true;
    m_render.state = Render_visuals_and_debug;
    m_render.Init();

    unsigned id = m_world.NewInstance();
    m_world.m_instances[id]->m_texture = m_world.m_assetMngr.LoadTexture("data/crab.png");

    m_world.background = m_world.m_assetMngr.LoadTexture("data/background.jpg");
    m_world.bg_position.Set();
    m_world.bg_size.Set(204.8,204.8);


    m_world.m_instances[id]->m_block.PushVertex( 1, 0);
    m_world.m_instances[id]->m_block.PushVertex( 4, 0);
    m_world.m_instances[id]->m_block.PushVertex( 5, 2);
    m_world.m_instances[id]->m_block.PushVertex(3.5, 4.5);
    m_world.m_instances[id]->m_block.PushVertex( 1.5, 4.5);
    m_world.m_instances[id]->m_block.PushVertex( 0, 2);

    bool valid = m_world.m_instances[id]->m_block.IsValid();

    m_world.NewUnit(id);



    BUG::AiPathfinder* pathfinder = &(m_world.m_aiMngr->m_pathfinder);
    
    // start counting
    srand(time.GetTime()*time.GetDT());

    int chunk = RAND_MAX*0.7;
    for (int i = 0; i < 205; i += 15)
    {
      for (int j = 0; j < 205; j += 15)
      {
        if (rand() < chunk)
        {
          BUG::AiPathNode* nNode = pathfinder->NewNode();
          nNode->m_pos.x = j + rand()%4 - 2; nNode->m_pos.y = i+ rand()%4 - 2;
          nNode->m_toll = MCG_SQRT((MCG::Real)(i*i + j*j));
        }
      }
    }
    DBUG__(DOUT << "nodes creation complete in\t" << time.GetDT() << "ms\n");

    pathfinder->GenerateNodeNetwork(8,25);
    DBUG__(DOUT << "node network complete in\t" << time.GetDT() << "ms\n");
  }

  //------------------------------------------------ Application::Update
  void Application::Update()
  {
    m_input.Update();

#if 1 // calculate new paths each frame
    if (m_world.sel_unit)
    {
      BUG::AiPathfinder* pfinder = &(BUG::AiManager::Instance()->m_pathfinder);
      MCG::Vector2 current = m_world.m_instances[m_world.sel_unit->m_instanceID]->m_T.GetTranslation();
      BUG::AiPathNode* start = pfinder->GetClosestNode(current.x, current.y);

      MCG::Vector2 mouse = m_input.mouse.p;
      mouse = m_render.Screen2World(mouse);

      BUG::AiPathNode* end = pfinder->GetClosestNode(mouse.x, mouse.y);

      m_world.lastPath = pfinder->CalculatePath(start,end);
    }
#endif // 0


    // if mouse is in border area, move camera in that direction
    unsigned border = 100, inner = 30;
    MCG::Real speed = 0.01, fast = 0.07;

    if (m_input.mouse.p.x < border)
    {
      if (m_input.mouse.p.x < inner)
        m_render.m_camera.TranslateManual(fast*dt,0,0);
      else
        m_render.m_camera.TranslateManual(speed*dt,0,0);
    }
    else if (m_input.mouse.p.x > m_render.m_width - border)
    {
      if (m_input.mouse.p.x > m_render.m_width - inner)
        m_render.m_camera.TranslateManual(-fast*dt,0,0);
      else
        m_render.m_camera.TranslateManual(-speed*dt,0,0);
    }

    if (m_input.mouse.p.y < border)
    {
      if (m_input.mouse.p.y < inner)
        m_render.m_camera.TranslateManual(0,-fast*dt,0);
      else
        m_render.m_camera.TranslateManual(0,-speed*dt,0);
    }
    else if (m_input.mouse.p.y > m_render.m_height - border)
    {
      if (m_input.mouse.p.y > m_render.m_height - inner)
        m_render.m_camera.TranslateManual(0,fast*dt,0);
      else
        m_render.m_camera.TranslateManual(0,speed*dt,0);
    }

    m_world.Update();
    m_render.Update();
  }

  //------------------------------------------------ Application::Close
  void Application::Close()
  {
    //m_world.m_aiMngr->m_pathfinder.SaveNodeNetwork("test.pnn", true);
  }
}
