/// \file Input.cpp
/// \date 2008/01/24
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#include "Input.h"
#include "Application.h"
#include "Instance.h"
#include "Unit.h"

// OpenGL Extension Wrangler - utilises entire openGL functionality
#include <GL/glew.h>
//#ifdef WIN32
//	#include <GL/wglew.h>
//#else
#ifdef LINUX
#include <GL/glxew.h>
#endif
//#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

#include <DebugUtl.h>
#include <MCG/Vector2.h>


namespace Roach
{

	//------------------------------------------------ Input::Input
  Input::Input()
  {
    EnableEvent(FRM_E_QUIT);
    EnableEvent(FRM_E_MOUSEBUTTONDOWN);
    EnableEvent(FRM_E_MOUSEBUTTONUP);
    EnableEvent(FRM_E_MOUSEMOTION);
    EnableEvent(FRM_E_KEYDOWN);
  } 

  //------------------------------------------------ Input::~Input
  Input::~Input()
  {
    
  }

  void Input::Quit()
  {
    Application::Instance()->status.Set(k_State_Exit);
    return;
  }

  //------------------------------------------------ Input::MouseButtonDown
  void Input::MouseButtonDown()
  {
    Application* app = Application::Instance();
    MCG::Vector2 m_pos = app->m_render.Screen2World(MCG::Vector2(mouse.p.x, mouse.p.y));
    app->m_world.ClickWorld(m_pos.x, m_pos.y, E->button.button - 1);
  }

  //------------------------------------------------ Input::MouseButtonUP
  void Input::MouseButtonUP()
  {
    Application* app = Application::Instance();
    MCG::Vector2 m_pos = app->m_render.Screen2World(MCG::Vector2(mouse.p.x, mouse.p.y));
    app->m_world.ClickWorld(m_pos.x, m_pos.y, E->button.button - 1);
  }

  //------------------------------------------------ Input::MouseMotion
  void Input::MouseMotion()
  {
#if 0
    if (mouse.buttonstate[FRM_M_BUTTON_1] && !Application::Instance()->m_world.HasUserSelection())
    {
      Application::Instance()->m_render.m_camera.TranslateManual ((MCG::Real)E->motion.xrel * 0.1f,(MCG::Real)E->motion.yrel * -0.1f,0,true);
    }
#endif // 0
  }

  //------------------------------------------------ Input::KeyDown
  void Input::KeyDown()
  {
    Roach::Application* App = Application::Instance();
    switch (E->key.keysym.sym)
    {
    case SDLK_ESCAPE:
      App->status.Set(k_State_Exit);
      break;

    case SDLK_F1:
      App->m_mode = App_Mode_Play;
      //App->m_render.state = Render_visuals;
      break;

    case SDLK_F2:
      App->m_mode = App_Mode_Units;
      //App->m_render.state = Render_visuals_and_debug;
      break;

    case SDLK_F3:
      App->m_mode = App_Mode_Instances;
      //App->m_render.state = Render_visuals_and_debug;
      break;

    case SDLK_F4:
      App->m_mode = App_Mode_PathNodes;
      //App->m_render.state = Render_visuals_and_debug;
      break;

    default:
      break;
    }
  }

  //------------------------------------------------ Input::Check_KeyState
  void Input::Check_KeyState()
  {
    Roach::Application* App = Application::Instance();

    Uint8 *_keystate = SDL_GetKeyState(NULL);

    if (_keystate[SDLK_LEFT])
      App->m_render.m_camera.TranslateManual( 0.05*App->dt,0,0);
    if (_keystate[SDLK_RIGHT])
      App->m_render.m_camera.TranslateManual(-0.05*App->dt,0,0);
    if (_keystate[SDLK_UP])
      App->m_render.m_camera.TranslateManual(0,-0.05*App->dt,0);
    if (_keystate[SDLK_DOWN])
      App->m_render.m_camera.TranslateManual(0, 0.05*App->dt,0);
  }
}