/// \file Render.cpp
/// \date 2008/01/24
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#include "Render.h"
#include "Application.h"
#include "Instance.h"
#include "Unit.h"
#include <map>

#include <FRM/FRM_Texture.h>
#include <FRM/FRM_Utl.h>

#include <BUG/AiManager.h>
#include <BUG/AiPathNode.h>

// OpenGL Extension Wrangler - utilises entire openGL functionality
#include <GL/glew.h>
//#ifdef WIN32
//	#include <GL/wglew.h>
//#else
#ifdef LINUX
#include <GL/glxew.h>
#endif
//#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

FRM::FTexture testTexture;

namespace Roach
{
  //------------------------------------------------ Render::Render
  Render::Render()
    : z_distance (0.1f), state (Render_visuals)
  {

  }

  //------------------------------------------------ Render::~Render
  Render::~Render()
  {

  }

  //------------------------------------------------ Render::Init_Projection
  void Render::Init_Projection()
  {
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //gluPerspective(45.0,(double)m_width/(double)m_height,0.01,1000.0);
    m_true_w = (double)m_width*z_distance;
    m_true_h = (double)m_height*z_distance;
    glOrtho(0,m_true_w,0,m_true_h,-1.0,100.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
  }

  //------------------------------------------------ Render::Init_Additional
  void Render::Init_Additional()
  {
    //SDL_ShowCursor(SDL_DISABLE); // hide
    //SDL_WM_GrabInput(SDL_GRAB_ON); // bind mouse to window limits

    SDL_WarpMouse((Uint16)(m_width*0.5),(Uint16)(m_height*0.5));

    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);

    glEnable (GL_LINE_SMOOTH);
    //glEnable(GL_DEPTH_TEST);

    //testTexture.Load("data/helloworld.jpg");

    //m_camera.Translate(MCG::Vector3(-10,-10,-10));
  }

  //------------------------------------------------ Render::Update
  void Render::Update()
  {
    Application* app = Application::Instance();
    glClear(GL_COLOR_BUFFER_BIT /*| GL_DEPTH_BUFFER_BIT*/);
    glClearColor(0,0,0,0);

    glLoadIdentity();
		#ifdef WIN32
    FRM_Utl_MultMatrix(m_camera.GetMatrix());
		#else
		MCG::Matrix mat = m_camera.GetMatrix();
		FRM_Utl_MultMatrix(mat);
		#endif

    glColor4f(1,1,1,1);
    FRM_Utl_DrawSprite(*app->m_world.background, app->m_world.bg_size.x, app->m_world.bg_size.y, FRM_Z_AXIS );


    if (state == Render_visuals || state == Render_visuals_and_debug)
      RenderVisuals();
    //if (state == Render_visuals_and_debug)
    //  glClear(GL_DEPTH_BUFFER_BIT);
    if (state == Render_debug || state == Render_visuals_and_debug)
      RenderDebug();

    //glClear(GL_DEPTH_BUFFER_BIT);

    glLoadIdentity();

    switch(app->m_mode)
    {
    case App_Mode_Units:
      glColor4f(0,0,1,1);
      break;
    case App_Mode_Instances:
      glColor4f(0,1,0,1);
    	break;
    case App_Mode_PathNodes:
      glColor4f(1,0,0,1);
    	break;
    case App_Mode_Play:
    default:
      glColor4f(0,0,0,0.2);
        break;
    }

    glBegin(GL_LINE_LOOP);
    glVertex3f(1,1,0);
    glVertex3f(1,m_true_h-1,0);

    glVertex3f(m_true_w-1,m_true_h-1,0);
    glVertex3f(m_true_w-1,1,0);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3f(1.3,1.3,0);
    glVertex3f(1.3,m_true_h-1.3,0);

    glVertex3f(m_true_w-1.3,m_true_h-1.3,0);
    glVertex3f(m_true_w-1.3,1.3,0);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3f(1.6,1.6,0);
    glVertex3f(1.6,m_true_h-1.6,0);

    glVertex3f(m_true_w-1.6,m_true_h-1.6,0);
    glVertex3f(m_true_w-1.6,1.6,0);
    glEnd();



    SDL_GL_SwapBuffers();
  }

  //------------------------------------------------ Render::RenderVisuals
  void Render::RenderVisuals()
  {
    Application* app = Application::Instance();
    //DrawInstances();
    glColor4f(1,1,1,1);
    std::map<unsigned,Instance*>::iterator it = app->m_world.m_instances.begin();
    std::map<unsigned,Instance*>::iterator itend = app->m_world.m_instances.end();
    for (;it!=itend;++it)
    {
      glPushMatrix();
			#ifdef WIN32
			FRM_Utl_MultMatrix((*it).second->m_T.GetMatrix());
			#else
			MCG::Matrix mat = (*it).second->m_T.GetMatrix();
			FRM_Utl_MultMatrix(mat);
			#endif
      FRM_Utl_DrawSprite( *((*it).second->m_texture), 5, 5, FRM_Z_AXIS );
      glPopMatrix();
    }

    if(app->m_world.sel_unit != 0)
    {
      MCG::Vector2 unit_pos = app->m_world.m_instances[app->m_world.sel_unit->m_instanceID]->m_T.GetTranslation();
      MCG::Vector2 drag_pos = Screen2World(app->m_input.mouse.p) + app->m_world.sel_offset;
      glPushMatrix();
			#ifdef WIN32
			FRM_Utl_MultMatrix(MCG::Matrix(drag_pos));
			#else
			MCG::Matrix mat (drag_pos);
			FRM_Utl_MultMatrix(mat);
			#endif
      //DrawInstances(app->m_world.sel_unit->m_instanceID,
      //  MCG::Colour4 (0.5,0.5,0.5,1) );
      glColor4f(1,1,1,0.5f);
      FRM_Utl_DrawSprite( *(app->m_world.m_instances[app->m_world.sel_unit->m_instanceID]->m_texture), 5, 5, FRM_Z_AXIS );
      glPopMatrix();
    }

  }

  //------------------------------------------------ Render::RenderDebug
  void Render::RenderDebug()
  {
    //glDisable(GL_TEXTURE_2D);

    Application* app = Application::Instance();
    FRM_Utl_DrawAxis(3);
    //glColor4f(0.5f,0.5f,0.5f,0.2f);
    //FRM_Utl_DrawGrid(50,FRM_Z_AXIS);

    //DrawAiDebug();

    std::map<unsigned,Instance*>::iterator iit = app->m_world.m_instances.begin();
    std::map<unsigned,Instance*>::iterator iendit = app->m_world.m_instances.end();
    BUG::AiBlock* block;
    glColor4f(0,1,0,0.5);
    for (;iit!=iendit;++iit)
    {
      glPushMatrix();
			#ifdef WIN32
			FRM_Utl_MultMatrix((*iit).second->m_T.GetMatrix());
			#else
			MCG::Matrix mat = (*iit).second->m_T.GetMatrix();
			FRM_Utl_MultMatrix(mat);
			#endif
      block = &(*iit).second->m_block;
      glBegin(GL_LINE_LOOP);
      glVertex3f(block->m_min.x,block->m_min.y,-2);
      glVertex3f(block->m_max.x,block->m_min.y,-2);

      glVertex3f(block->m_max.x,block->m_max.y,-2);
      glVertex3f(block->m_min.x,block->m_max.y,-2);
      glEnd();


      glBegin(GL_TRIANGLE_FAN);
      std::vector<BUG::Vector>::iterator vit = (*iit).second->m_block.vertices.begin();
      std::vector<BUG::Vector>::iterator vendit = (*iit).second->m_block.vertices.end();
      for(;vit!=vendit;++vit)
      {
        glVertex3f((*vit).x,(*vit).y,-2);
      }
      glEnd();

      glPopMatrix();
    }


    BUG::AiManager* aiMngr = BUG::AiManager::Instance();
    std::vector<BUG::AiPathNode*>::iterator it = aiMngr->m_pathfinder.m_pathnodes.begin();
    std::vector<BUG::AiPathNode*>::iterator endit = aiMngr->m_pathfinder.m_pathnodes.end();

    glBegin(GL_LINES);
    for (;it!=endit;++it)
    {
      glColor4f(1,0,0,1);
      glVertex3f((*it)->m_pos.x + 1,(*it)->m_pos.y - 1,-1);
      glVertex3f((*it)->m_pos.x - 1,(*it)->m_pos.y + 1,-1);

      glVertex3f((*it)->m_pos.x + 1,(*it)->m_pos.y + 1,-1);
      glVertex3f((*it)->m_pos.x - 1,(*it)->m_pos.y - 1,-1);

      glColor4f(1,1,0,1);
      std::vector<BUG::AiPathNode*>::iterator s_it = (*it)->m_successors.begin();
      for(;s_it!=(*it)->m_successors.end();++s_it)
      {
        glVertex3f(  (*it)->m_pos.x,  (*it)->m_pos.y, -2);
        glVertex3f((*s_it)->m_pos.x,(*s_it)->m_pos.y, -2);
      }

    }
    glEnd();

    BUG::AiPathNode* pn = app->m_world.sel_pathnode;
    if (pn)
    {
      MCG::Vector2 m_pos = Screen2World(app->m_input.mouse.p);
      glBegin(GL_LINES);
      if (app->m_input.mouse.buttonstate[FRM_M_BUTTON_1])
      {
        glColor4f(1,0,0,1);
        glVertex3f(m_pos.x + 1,m_pos.y - 1,-1);
        glVertex3f(m_pos.x - 1,m_pos.y + 1,-1);
      
        glVertex3f(m_pos.x + 1,m_pos.y + 1,-1);
        glVertex3f(m_pos.x - 1,m_pos.y - 1,-1);
      
        glColor4f(1,1,0,1);
        std::vector<BUG::AiPathNode*>::iterator s_it = pn->m_successors.begin();
        for(;s_it!=pn->m_successors.end();++s_it)
        {
          glVertex3f(m_pos.x, m_pos.y, -2);
          glVertex3f((*s_it)->m_pos.x, (*s_it)->m_pos.y, -2);
        }
      }
      else if (app->m_input.mouse.buttonstate[FRM_M_BUTTON_3])
      {
        glColor4f(1,1,0,0.5);
        glVertex3f(pn->m_pos.x,pn->m_pos.y,-2);
        glVertex3f(m_pos.x,m_pos.y,-2);
      }
      glEnd();
    }

    DrawAiPath(&(app->m_world.lastPath));
  }

  //------------------------------------------------ Render::DrawAiPath
  void Render::DrawAiPath( BUG::AiPath* path )
  {
    glBegin(GL_LINE_STRIP);
    glColor4f(1,0,0,1);
    for (unsigned i = 0; i < path->size(); ++i)
      glVertex3f((*path)[i]->m_pos.x, (*path)[i]->m_pos.y, 0);
    glEnd();
  }
}