/// \file World.cpp
/// \date 2008/02/15
/// \author Peter Dodds
/// \brief 
/// \note Copyright (C) 2008 - All Rights Reserved
//------------------------------------------------------------------------------

#include "World.h"
#include "Instance.h"
#include "Unit.h"

#include <BUG/AiMove.h>
#include <BUG/AiPath.h>
#include <BUG/AiPathNode.h>
#include <BUG/AiPathfinder.h>

#include <assert.h>

namespace Roach
{
  //------------------------------------------------ World::World
  World::World()
    : sel_unit (0), sel_pathnode (0)
  {
    BUG::AiManager::Make();
    m_aiMngr = BUG::AiManager::Instance();
  }

  //------------------------------------------------ World::~World
  World::~World()
  {
    std::map<unsigned,Instance*>::iterator iit = m_instances.begin();
    for (;iit!=m_instances.end();++iit)
    {
      delete (*iit).second;
    }
    std::map<unsigned,Unit*>::iterator uit = m_units.begin();
    for (;uit!=m_units.end();++uit)
    {
      delete (*uit).second;
    }

    BUG::AiManager::Delete();
  }

  //------------------------------------------------ World::NewInstance
  unsigned World::NewInstance()
  {
    Instance* n_inst = new Instance;
    m_instances[n_inst->GetID()] = n_inst;
    return n_inst->GetID();
  }

  //------------------------------------------------ World::DeleteInstance
  bool World::DeleteInstance( unsigned id )
  {
    if(m_instances.find(id) == m_instances.end())
      return false;

    delete m_instances[id];
    m_instances.erase(id);
    return true;
  }

  //------------------------------------------------ World::NewUnit
  unsigned World::NewUnit( unsigned instanceId )
  {
    assert (m_instances.find(instanceId) != m_instances.end());

    Unit* n_U = new Unit;
    m_units[n_U->GetID()] = n_U;
    n_U->m_instanceID = instanceId;

    return n_U->GetID();
  }

  //------------------------------------------------ World::DeleteUnit
  bool World::DeleteUnit( unsigned id, bool andInstance /*= false*/ )
  {
    if(m_units.find(id) == m_units.end())
      return false;

    if (andInstance && DeleteInstance(m_units[id]->m_instanceID) == false)
      return false;

    delete m_units[id];
    m_units.erase(id);
    return true;
  }

  //------------------------------------------------ World::ClickWorld
  void World::ClickWorld( MCG::Real _x, MCG::Real _y, const unsigned button /*= 0*/ )
  {
    Application* app = Application::Instance();

    switch(app->m_mode)
    {
    case App_Mode_Units:
      {

      }
    	break;

    case App_Mode_Instances:
      {

      }
    	break;

    case App_Mode_PathNodes:
      {
        if (button == 0 && sel_pathnode)
        {
          sel_pathnode->m_pos.x = _x;
          sel_pathnode->m_pos.y = _y;
          sel_pathnode = 0;
          return;
        }

        // test if click hits aipathnode
        std::vector<BUG::AiPathNode*>::iterator pn_it = m_aiMngr->m_pathfinder.m_pathnodes.begin();
        std::vector<BUG::AiPathNode*>::iterator pn_end = m_aiMngr->m_pathfinder.m_pathnodes.end();
        for (;pn_it!=pn_end;++pn_it)
        {
          if (_x > (*pn_it)->m_pos.x-2 && _y > (*pn_it)->m_pos.y-2 &&
            _x < (*pn_it)->m_pos.x+2 && _y < (*pn_it)->m_pos.y+2)
          {
            if (!sel_pathnode)
            {
              if (app->m_input.keystate[FRM_K_LCTRL])
              {
                if (button == 0)
                  m_aiMngr->m_pathfinder.DeleteNode(*pn_it);
              }
              else
                sel_pathnode = (*pn_it);
            }
            else
            {
              if (button == 2)
              {
                // link selected node to hit node
                m_aiMngr->m_pathfinder.LinkNodes(sel_pathnode,*pn_it);
                sel_pathnode = 0;
              }
            }
            return;
          }
        }

        if (app->m_input.keystate[FRM_K_LSHIFT])
        {
          if (button == 0)
          {
            sel_pathnode = m_aiMngr->m_pathfinder.NewNode();
            sel_pathnode->m_pos.x = _x;
            sel_pathnode->m_pos.y = _y;
          }
        }
      }
      break;

    case App_Mode_Play:
    default:
      {

      }
      break;
    }

    
    // test if click hits unit's instance bouding box
    std::map<unsigned,Unit*>::iterator it = m_units.begin();
    for (;it!=m_units.end() && button == 0;++it)
    {
      Instance* inst = m_instances[(*it).second->m_instanceID];
    
      if (inst->m_block.IsPointInside(_x,_y))
      {
        if (button == 0 /*left*/)
        {
          if (sel_unit == 0)
          {
            sel_unit = (*it).second;
            sel_offset = inst->m_T.GetTranslation()
              - app->m_render.Screen2World(app->m_input.mouse.p);
          }
          else
          {
            sel_unit = 0;
            sel_offset.Set();
          }
          return;
        }
        else
          break;
      }
    }
    
    if (button == 0 && sel_unit)
    {
      BUG::AiPathfinder* pfinder = &(BUG::AiManager::Instance()->m_pathfinder);
      MCG::Vector2 current = m_instances[sel_unit->m_instanceID]->m_T.GetTranslation();
      BUG::AiPathNode* start = pfinder->GetClosestNode(current.x, current.y);
      BUG::AiPathNode* end = pfinder->GetClosestNode(_x, _y);
    
      sel_unit->Ai_UsePathAsMoves(lastPath = pfinder->CalculatePath(start,end));
      BUG::AiMove_Goto* _goto = (BUG::AiMove_Goto*)sel_unit->Ai_AddMove(BUG_Move_Goto);
      _goto->goal.x = _x; _goto->goal.y = _y;
    
      sel_unit = 0;
      sel_offset.Set();
      return;
    }

  }

  //------------------------------------------------ World::Update
  void World::Update()
  {
    m_aiMngr->Update(Application::Instance()->dt);

    std::map<unsigned,Instance*>::iterator it = m_instances.begin();
    std::map<unsigned,Instance*>::iterator endit = m_instances.end();
    for (;it!=endit;++it)
      (*it).second->Update();
  }
}